﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;
using System.Diagnostics;

namespace EXIFLOG
{
    /// <summary>
    /// The metadata for a set of files, obtained by shelling out to exiftool.exe,
    /// which must reside in the application .exe folder.
    /// </summary>

    public static class MetadataCollector
    {

        /// <summary>
        /// Obtains the metadata.
        /// </summary>
        /// <param name="filenames">
        /// Full paths to all the files.
        /// </param>
        public static List<ImageMetadata> Run(IEnumerable<string> filenames)
        {
            // Make sure exiftool.exe is available
            if (!File.Exists("exiftool.exe"))
            {
                throw new ApplicationException("EXIFLOG requires a copy of exiftool.exe (with that name) in the same folder as EXIFLOG.exe.");
            }

            // Create a temporary file to hold the filenames
            var tempFileName = Path.GetTempFileName();
            File.WriteAllLines(tempFileName, filenames.ToArray());
            try
            {
                RunExifTool(tempFileName);
            }
            finally
            {
                File.Delete(tempFileName);
            }

            return ParseExiftoolOutput();
        }

        private static List<string> CapturedOutput;

        /// <summary>
        /// Run Exiftool and put its output into CapturedOutput.
        /// </summary>
        /// <param name="argFileName"></param>
        private static void RunExifTool(string argFileName)
        {
            CapturedOutput = new List<string>();

            // Process start parameters
            var psi = new ProcessStartInfo("exiftool.exe", "-q -t -@ " + argFileName);
            psi.CreateNoWindow = true;
            psi.RedirectStandardOutput = true;
            psi.RedirectStandardError = true;
            psi.UseShellExecute = false;

            // The process itself and its event handlers
            var p = new Process() { StartInfo = psi };
            p.OutputDataReceived += new DataReceivedEventHandler(OutputHandler);
            p.ErrorDataReceived += new DataReceivedEventHandler(OutputHandler);

            // Start the process, and immediately start harvesting 
            // stdout and stderr lines to the event handler
            p.Start();
            p.BeginOutputReadLine();
            p.BeginErrorReadLine();

            // Do not return until the process is finished.
            p.WaitForExit();

        }

        // Event handler to harvest output from the command process
        // and send it to CapturedOutput
        static void OutputHandler(object sendingProcess, DataReceivedEventArgs outLine)
        {
            CapturedOutput.Add(outLine.Data);
        }

        /// <summary>
        /// Reads captured output from Exiftool, turns it into a dictionary of labels to values.
        /// </summary>
        static List<ImageMetadata> ParseExiftoolOutput()
        {
            var result = new List<ImageMetadata>();

            var metadata = new ImageMetadata();

            foreach (string line in CapturedOutput)
            {
                if (String.IsNullOrEmpty(line)) continue;

                // Split at the tab
                string[] fields = line.Split('\t');
                if (fields.Length != 2) continue;

                // Are we starting a new file?
                if (fields[0] == "File Name")
                {
                    if (metadata.ContainsKey("File Name"))
                    {
                        result.Add(metadata);
                        metadata = new ImageMetadata();
                    }
                }

                // Save this field
                metadata[fields[0]] = fields[1];
            }

            // Last one
            result.Add(metadata);

            return result;
        }
    }
}